#include "stdafx.h"
#include "kilib.h"


//-------------------------------------------------------------------------


#ifdef NO_NEED_FOR_C_RUNTIME

	static HANDLE s_hProcessHeap;

	static bool init_heap()
	{
		return (NULL!=(s_hProcessHeap=::GetProcessHeap())) ? true : false;
	}

	void* __cdecl operator new( size_t siz )
	{
		return ::HeapAlloc( s_hProcessHeap, 0, siz );
	}

	void __cdecl operator delete( void* ptr )
	{
		::HeapFree( s_hProcessHeap, 0, ptr );
	}

	extern "C" int __cdecl _purecall()
	{
		return 0;
	}

	#pragma comment(linker, "/entry:\"StartUp\"")
	#pragma comment(linker, "/opt:\"nowin98\"")

#else // if NEED_C_RUNTIME

	static bool init_heap()
	{
		return true;
	}

	int APIENTRY WinMain( HINSTANCE, HINSTANCE, LPSTR, int )
	{
		kilib::App::StartUp();
		return 0;
	}

#endif


//-------------------------------------------------------------------------


namespace kilib {


//-------------------------------------------------------------------------


App& App::GetInstance()
{
	return (*s_pApp);
}

HINSTANCE App::hinst() const
{
	return m_hInstance;
}

HWND App::hwnd() const
{
	return m_pMainWnd ? m_pMainWnd->hwnd() : NULL;
}

Wnd* App::wnd() const
{
	return m_pMainWnd;
}

void App::SetMainWnd( Wnd* wnd )
{
	m_pMainWnd = wnd;
}


//-------------------------------------------------------------------------


App::App()
{
	m_fLoadedModule = 0;
	m_pMainWnd      = NULL;
	m_hInstance     = ::GetModuleHandle( NULL );
	::SHGetMalloc( &m_pShellAllocator );
}

App::~App()
{
	if( m_fLoadedModule & COM )
	{
		::CoUninitialize();
	}
	if( m_fLoadedModule & OLE )
	{
		::OleUninitialize();
	}
	m_pShellAllocator->Release();
}


//-------------------------------------------------------------------------


void App::InitModule( ulong what )
{
	if( (what & CTL) && !(m_fLoadedModule & CTL) )
	{
		::InitCommonControls();
	}
	if( (what & COM) && !(m_fLoadedModule & COM) )
	{
		::CoInitialize( NULL );
	}
	if( (what & OLE) && !(m_fLoadedModule & OLE) )
	{
		::OleInitialize( NULL );
	}
	m_fLoadedModule |= what;
}

void App::ShellFree( void* ptr ) const
{
	m_pShellAllocator->Free( ptr );
}


//-------------------------------------------------------------------------


App* App::s_pApp = NULL;

void APIENTRY App::StartUp()
{
	int err = 0;

	if( init_heap() )
	{
		util::InitUtilLib();
		Wnd::InitWndLib();

		if( CreateInstance() )
		{
			app().Run();
		}
		else
		{
			err = 2;
		}
	}
	else
	{
		err = 1;
	}

	Exit( err );
}

void App::Exit( int endcode )
{
	delete s_pApp;
	Wnd::FinishWndLib();
	::ExitProcess( endcode );
}


//-------------------------------------------------------------------------


};     // namespace kilib
